/*
  main.cpp - Software USB1.1 Device Library (Bootloader for Micronucleus)
  
  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include <avr/io.h>
#include <avr/wdt.h>
#include "swusb.h"

static void reset(void) __attribute__((naked, section(".reset")));
static void reset(void)
{
  #if FLASHEND < 0x2000
    asm volatile ("rjmp __vectors");
  #else
    asm volatile ("jmp __vectors");
  #endif
}

uint8_t mcusr;

int main(void)
{
#if defined(MCUSR)
  mcusr = MCUSR;
  MCUSR = 0;
  wdt_disable();
  /* not hardware reset ? */
  if (mcusr == 0)
  {
    wdt_enable(WDTO_15MS);
    while (1) ;
  }
#else
  mcusr = RSTCTRL_RSTFR;
  RSTCTRL_RSTFR = 0xFF;
  wdt_disable();
  /* not hardware reset ? */
  if (mcusr == 0)
    RSTCTRL_SWRR = RSTCTRL_SWRE_bm;
  if ((CLKCTRL_MCLKCTRLA & CLKCTRL_CLKSEL_gm) == (CLKCTRL_CLKSEL_OSC20M_gc << CLKCTRL_CLKSEL_gp))
  {
    CPU_CCP = CCP_IOREG_gc;
    CLKCTRL_MCLKCTRLB = 0; // no prescaler
  }
#endif
  SWUSB::boot();
  reset();
}
