/*
  swusb.h - Software USB1.1 Device Library (Bootloader for Micronucleus)
  
  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <stdint.h>
#include <stdbool.h>

class SWUSB
{
  public:

    static void connect(void);
    static void disconnect(void);
    static void boot(void);
    static bool connected(void);

  protected:

    typedef struct __attribute__((packed))
    {
      uint8_t rcpt: 5;
      uint8_t type: 2;
      uint8_t dir : 1;
    } request_type_t;

    typedef struct __attribute__((packed))
    {
      request_type_t bmRequestType;
      uint8_t  bRequest;
      uint16_t wValue;
      uint16_t wIndex;
      uint16_t wLength;
    } request_t;

    static void reset(void);
    static void control(const uint8_t *buf, uint8_t len);
    static void setup(const request_t *req);
    static void command(const request_t *req);
    static void command(void);
    static void startApp(void);
    static void handshake(uint8_t pid);
    static void response(const uint8_t *buf, uint8_t len, bool pgm = false);
    static void buildpacket(void);
    static void transmit(void);
    static void poll(uint8_t *buf, uint8_t len) __attribute__((noinline, optimize(1)));
    static uint8_t read(uint8_t *buf, uint8_t len) __attribute__((noinline, optimize(1), naked));
    static void write(const uint8_t *buf, uint8_t len) __attribute__((noinline, optimize(1), naked));
    static uint16_t crc16(uint8_t *dst, const uint8_t *src, uint8_t len, bool pgm) __attribute__((noinline, optimize(1), naked));
    static void usbBusArbitration(void) __attribute__((noinline, optimize(1), naked));
    static uint16_t usbMeasureFrameLength(void) __attribute__((noinline, optimize(1), naked));
    static bool calibration(void) __attribute__((noinline));
};
